.title "CNRAM Loader"
;2005/12/01
;Written by Quietust
;Version 1.0

		;vectors for standard system calls
.org $0200
write_byte:	.block 3	;sends byte in accumulator to PC
baton:		.block 3	;spins a baton on the LCD, overwrites A
chk_vram:	.block 3	;checks for CHR RAM and returns result in Z flag, overwrites A
chk_wram:	.block 3	;checks for writable PRG-RAM at $6000-$7FFF and returns result in Z flag, overwrites A+X+Y
wr_ppu:		.block 3	;sets VRAM address upper byte to A, lower byte to $00
read_byte:	.block 3	;reads byte from PC and stores in accumulator, overwrites X
init_crc:	.block 3	;initializes CRC32 value, overwrites A
do_crc:		.block 3	;updates CRC32 with value in accumulator, overwrites X
finish_crc:	.block 3	;finishes CRC32, overwrites A+X

.org $0080
crc0:		.block 1	;CRC32 value
crc1:		.block 1
crc2:		.block 1
crc3:		.block 1

.org $00C0			;User variables
temp1:
temp1_lo:	.block 1
temp1_hi:	.block 1

temp2:
temp2_lo:	.block 1
temp2_hi:	.block 1

romsiz:		.block 1
banknum:	.block 1
numpages:	.block 1
pagenum:	.block 1

.org $0380
.db "CNRAM Loader",0

.fill $0400-*,$00			;all plugins must reside at $0400

reset:	LDA #$00
	STA $4803	;input mode

	JSR read_byte	;get CHR ROM size, in 8KB banks
	STA romsiz
	ASL A
	ASL A
	ASL A
	ASL A
	ASL A
	STA numpages	;convert to number of 256-byte blocks

	LDY #$00
	STY temp1	;init CHR ROM pointer
	STY banknum	;and CHR bank number

	LDX romsiz
	BEQ nochr
	DEX
	BEQ chr8k
	DEX
	BEQ chr16k
	BNE chr32k

nochr:	JMP do_prg
chr8k:	JMP chr8k1

;-----------------------------------------------------------------------

chr32k:
chr32k1:		;easy case - 32KB of CHR data
	LDA $2002
	TYA
	JSR wr_ppu	;start at $0000

	LDA banknum
	STA $8000	;set CHR bank

	LDA #$20
	STA pagenum
chr32k2:
	JSR read_byte
	STA $2007
	INY
	BNE chr32k2	;copy 256 bytes
	DEC numpages
	BEQ chr32k3	;(stopping if we run out)
	DEC pagenum
	BNE chr32k2	;copy 32 of these blocks (8KB)
	INC banknum	;and then go to the next bank
	JMP chr32k1

chr32k3:
	JMP do_prg	;all done

;-----------------------------------------------------------------------

chr16k:
chr16k1:		;Complicated case - 16KB of CHR data
	JSR read_byte
	STA chrbuffer,Y
	INY
	BNE chr16k1	;buffer 256 bytes

	LDA $2002
	LDA temp1
	JSR wr_ppu
	LDA banknum
	STA $8000	;set VRAM address in lower 16KB

chr16k2:
	LDA chrbuffer,Y
	STA $2007
	INY
	BNE chr16k2	;and write it in

	LDA $2002
	LDA temp1
	JSR wr_ppu
	LDA banknum
	ORA #$02
	STA $8000	;and repeat for the upper 16KB

chr16k3:
	LDA chrbuffer,Y
	STA $2007
	INY
	BNE chr16k3

	DEC numpages
	BEQ chr16k4
	INC temp1
	LDA temp1
	CMP #$20
	BNE chr16k1	;repeat for the next page
	INC banknum	;once we hit $2000, reset to $0000
	LDA #$00	;and jump to the upper 8KB (of each segment)
	STA temp1
	BEQ chr16k1

chr16k4:
	JMP do_prg	;all done

;-----------------------------------------------------------------------

chr8k1:			;Slightly easier case - 8KB of CHR data
	JSR read_byte
	STA chrbuffer,Y
	INY
	BNE chr8k1	;buffer 256 bytes

	LDA #$00
	STA banknum	;clear bank number
	LDA #$04
	STA pagenum	;4 banks to load
chr8k2:
	LDA $2002
	LDA temp1
	JSR wr_ppu
	LDA banknum
	STA $8000	;set VRAM address
chr8k3:
	LDA chrbuffer,Y
	STA $2007
	INY
	BNE chr8k3	;copy 256 bytes into current bank
	INC banknum	;repeat for next bank
	DEC pagenum
	BNE chr8k2
	
	DEC numpages
	BEQ chr8k4

	INC temp1	;increment VRAM addr
	JMP chr8k1

chr8k4:
;	JMP do_prg

;-----------------------------------------------------------------------

do_prg:
	JSR read_byte	;get PRG ROM size, in 16KB banks
	STA romsiz
	ASL A
	ASL A
	ASL A
	ASL A
	ASL A
	ASL A
	STA numpages	;convert to number of 256-byte blocks

	LDA #$00
	STA temp1_lo
	STA temp2_lo
	LDA #$80
	STA temp1_hi
	LDA #$C0
	STA temp2_hi

	LDY #$00
	LDX romsiz
	DEX
	BEQ prg16
;	BNE prg32

prg32:
	JSR read_byte
t1:	STA $8000,Y
	INY
	BNE prg32
	INC t1+2	;temp1_hi
	DEC numpages
	BNE prg32
	JMP runwait1

prg16:
	JSR read_byte
t2:	STA $8000,Y
t3:	STA $C000,Y
	INY
	BNE prg16
	INC t2+2	;temp1_hi
	INC t3+2	;temp2_hi
	DEC numpages
	BNE prg16
;	JMP runwait1

runwait1:
	JSR read_byte
	CMP #$55
	BNE runwait1

runwait2:
	JSR read_byte
	CMP #$AA
	BNE runwait2

	LDA #$FC
	STA $4800	;unmap BIOS
	JMP ($FFFC)	;and jump into game

.fill $0700-*,$FF	;fill to 768 bytes
chrbuffer:		;and fill remaining 256 bytes with a buffer
.fill $0800-*,$FF

;ROMstring (optional, null-terminated)

.end
